-module(gatherer).
-behaviour(gen_server).

-export([start_link/2, init/1, terminate/2]).
-export([handle_info/2, handle_cast/2, handle_call/3]).
-export([code_change/3]).

-record(log, {file, error_regex}).

start_link(File, ErrorRegex) ->
    gen_server:start_link(?MODULE, [#log{file = File, error_regex = ErrorRegex}], []).

init([Log]) ->
    {ok, [off, Log]}.

handle_info({log_line, Text}, [off, Log = #log{error_regex = ErrorRegex}]) ->
    case isError(Text, ErrorRegex) of
        true ->
            {ok, Timer} = timer:send_after(1000, {timeout}),
            {ok, SafeTimer} = timer:send_after(30000, {timeout}),
            {noreply, [on, Log, Text, Timer, SafeTimer]};
        false -> {noreply, [off, Log]}
    end;
handle_info({log_line, Text}, [on, Log = #log{error_regex = ErrorRegex}, Error, Timer, SafeTimer]) ->
    case isError(Text, ErrorRegex) of
        true ->
            timer:cancel(Timer),
            {ok, NewTimer} = timer:send_after(1000, {timeout}),
            {noreply, [on, Log, Error ++ Text, NewTimer, SafeTimer]};
        false ->
            {noreply, [on, Log, Error ++ Text, Timer, SafeTimer]}
    end;
handle_info({timeout}, [on, Log = #log{file = File}, Error, Timer, SafeTimer]) ->
    timer:cancel(Timer),
    timer:cancel(SafeTimer),
    mailer ! {error, File, Error},
    {noreply, [off, Log]}.

handle_cast(_Msg, State) ->
    {noreply, State}.

handle_call(_Request, _From, State) ->
    {noreply, State}.

terminate(_Reason, _State) ->
    shutdown.

code_change(_OldVsn, State, _Extra) ->
    {ok, State}.

isError(Text, ErrorRegex) ->
    case re:run(Text, ErrorRegex) of
        {match, _} ->
            true;
        nomatch ->
            false
    end.
