-module(watcher).
-behaviour(gen_server).

-export([start_link/2, init/1, terminate/2]).
-export([handle_info/2, handle_cast/2, handle_call/3]).
-export([code_change/3]).

start_link(SupPid, File) ->
    gen_server:start_link(?MODULE, [SupPid, File], []).

init([SupPid, File]) ->
    timer:sleep(1000),
    config ! {watcher_init, File},
    Cmd = "/usr/bin/tail -n0 --follow=name " ++ File,
    Port = open_port({spawn, Cmd}, [stderr_to_stdout, exit_status, binary]),
    {ok, [File, SupPid, Port]}.

handle_info(Msg, [File, SupPid, Port]) ->
    case Msg of
        {Port, {data, Text}} ->
            GathererPid = gatherer_pid(SupPid),
            GathererPid ! {log_line, binary_to_list(Text)},
            {noreply, [File, SupPid, Port]};
        {Port, {exit_status, _Status}} ->
            {stop, tail_exit, [File]}
    end.

handle_cast(_Msg, State) ->
    {noreply, State}.

handle_call(_Request, _From, State) ->
    {noreply, State}.

terminate(_Reason, [File]) ->
    config ! {watcher_terminate, File},
    shutdown.

code_change(_OldVsn, State, _Extra) ->
    {ok, State}.

gatherer_pid(SupPid) ->
    hd([Pid || {Id, Pid, _, _} <- supervisor:which_children(SupPid), Id == gatherer]).
